<?php

interface READERInterface {
    public function __construct(&$props);

    public function GetDatabaseList($flags = 0);
    public function GetGroupList($flags = 0);
    public function GetGroupInfo(LOGGROUP $grp = NULL, $flags = 0);
    public function GetItemList(LOGGROUP $grp = NULL, MASK $mask = NULL, $flags = 0);
    public function GetExperimentList($flags = 0);
    public function GetMaskList(LOGGROUP $grp = NULL, $flags = 0);
    
    public function CreateGroup(array &$ginfo = NULL, $flags = 0);
    public function CreateInterval(LOGGROUP $group = NULL, array &$iinfo = NULL, $flags = 0);
    public function CreateMask(LOGGROUP $grp = NULL, array &$minfo = NULL, $flags = 0);
    
    public function GetGroups();
    public function GetGroupSize(LOGGROUP $grp = NULL, $flags = 0);
    
    public function ImportUnixTime($unix_time);
    public function ExportUnixTime($db_time);

    public function HaveData(LOGGROUP $grp = NULL, $from = false, $to = false);
    public function GetRawData(LOGGROUP $grp = NULL, $from = false, $to = false, DATAFilter $filter = NULL, &$filter_data = NULL);
    public function GetRawAlarms(LOGGROUP $grp = NULL, $from = false, $to = false, DATAFilter $filter = NULL, &$filter_data = NULL);

    public function GetData(LOGGROUP $grp = NULL, $from = false, $to = false, MASK $mask = NULL, $resample = 0, $limit = 0);
    public function GetAlarms(LOGGROUP $grp = NULL, $from = false, $to = false, MASK $mask = NULL, $mode = 0, $severity = false, $limit = false);

    public function PushData(LOGGROUP $grp, $time, $data);
    public function RemoveData(LOGGROUP $grp, $time);
    
    public function Backup(&$binfo);
}


abstract class READER implements READERInterface {
 var $req;
 var $server;
 var $opts;

 var $srvid, $dbname;
 var $start_date;
 var $end_date;

 var $no_default_time_moudle;	// Setting to false prevents creation default timemodule
 var $time_format;
 var $time_zone;
 var $time_module;

 var $gmt_timezone;
 
 var $group_class;
 
 var $groups;			// Per group options, configs, etc...
 
 const BACKUP_FULL = 1;

 const ALARM_MODE_FULL_LIST = 1;
 const ALARM_MODE_ACTIVE_LIST = 2;

 function __construct(&$props) {
    if ($props instanceof REQUEST) {
	$this->req = &$props;
	$this->server = $this->req->GetServerInfo(REQUEST::NEED_INFO);
	
	$this->srvid = $props->props['db_server'];
    } else {
	$this->req = false;
	$this->server = &$props;

	$this->srvid = false;
    }
    
    if (isset($this->server['database']))
	$this->dbname = $this->server['database'];
    else 
	$this->dbname = false;
	
    $this->opts = new OPTIONS($props);

    $date_limit = $this->opts->Get('date_limit');
    if ($date_limit) {
	if (is_array($date_limit)) {
	    if (is_int($date_limit[0])) $this->start_date = $date_limit[0];
	    else $this->start_date = strtotime($date_limit[0]);
	    if (is_int($date_limit[1])) $this->end_date = $date_limit[1];
	    else $this->end_date = strtotime($date_limit[1]);
	} else {
	    if (is_int($date_limit)) $this->start_date = $date_limit;
	    else $this->start_date = strtotime($date_limit);
	    $this->end_date = false;
	}
    } else {
	$this->start_date = false;
	$this->end_date = false;
    }

    $this->gmt_timezone = new DateTimeZone("GMT");
    
    $time_module = $this->opts->Get('time_module');
    if ($time_module) {
	if (!include_once("time/" . strtolower($time_module) . '.php')) {
	    throw new ADEIException(translate("Unsupported time module is configured: \"%s\"", $time_module));
	}
	
	$time_options = $this->opts->Get('time_options');
	$cl = strtoupper($time_module);
	$this->time_module = new $cl($this, $time_options);
    } else {
	$time_format = $this->opts->Get("timeformat");
	if ($time_format) $this->time_format = $time_format;
	else if (!$this->no_default_time_module) $this->time_format = "U.u";
	else $this->time_format = false;

        $time_zone = $this->opts->Get("timezone");
	if ($time_zone) $this->time_zone = new DateTimeZone($time_zone);
	else $this->time_zone = new DateTimeZone("GMT");

	if ($this->time_format) {
	    $this->time_module = new READERTime($this, array(
		'format' => $this->time_format,
	        'timezone' => $this->time_zone
	    ));
	}
    }
    
    $this->group_class = false;
    $this->groups = array();
 }

 function ImportTime(DateTime $dt) {
    return $this->time_module->ImportTime($dt);
 }

 function ExportTime($db_time) {
    return $this->time_module->ExportTime($db_time);
 }

 function ImportUnixTime($unix_time) {
    return $this->time_module->ImportUnixTime($unix_time);
 }
 
 function ExportUnixTime($db_time) {
    return $this->time_module->ExportUnixTime($db_time);
 }
 
 function GetSourceTitle() {
    $server = $this->req->GetServerConfig();

    $title = $server['title'];
    if ($this->dbname) {
	$dblist = $this->GetDatabaseList();
	$title .= " - " . $dblist[$this->dbname]["name"];
    }
    return $title;
 }
 
 function GetGroupTitle(LOGGROUP $grp = NULL, $just_group = false) {
    $grp = $this->CheckGroup($grp);
    $groupinfo = $this->GetGroupInfo($grp);

    if ($just_group) return $groupinfo["name"];
    else {
	$title = $this->GetSourceTitle(); 
	return $title . " - " . $groupinfo["name"];
    }
 }

 function GetDatabaseList($flags = 0) {
    if ($this->req)
	$server = $this->req->GetServerConfig();
    else
	throw new ADEIException(translate("The data source server is not specified"));

    $dblist = array();
    foreach ($server['database'] as $name) {
	$dblist[$name] = array(
	    'name' => $name
	);
    }
    
    return $dblist;
 }

 function GetGroupList($flags = 0) {
    return $this->GetGroupInfo($gr=NULL, $flags);
 }

 function GetAlarmList(LOGGROUP $grp = NULL, MASK $mask = NULL, $flags = 0) {
    return array();
 }

 function GetExperimentList($flags = 0) {
    return array();
 }

 protected function AppendStandardMasks(LOGGROUP $grp, array &$mask, $flags = 0) {
    if (!$mask['all']) {
	$mask['all'] = array(
	    'id' => "all",
	    "name" => _("All")
	);
    
	if ($flags&REQUEST::NEED_INFO) {
	    $items = $this->GetItemList($grp);
	    foreach (array_keys($items) as $i) {
		if ($i) $all .= "," . $i;
		else $all = $i;
	    }
	    unset($items);
	
	    $mask['all']['mask'] = $all;
	} 
    }
 }
 
 function GetMaskList(LOGGROUP $grp = NULL, $flags = 0) {
    $grp = $this->CheckGroup($grp);
    
    $mask = array();
    $this->AppendStandardMasks($grp, $mask, $flags);
    return $mask;
 }

 protected function AddAxisInfo(LOGGROUP $grp, array &$items) {
    $axis_opt = $this->req->GetGroupOption("axis", $grp);
    if ($axis_opt) {
	foreach ($items as &$item) {
	    if (is_array($axis_opt)) {
    		foreach ($axis_opt as $re => $axis) {
		    if (!strcmp($re,"*")) {
			$item['axis'] = $axis;
		    } else {
			if ($item['uid']) {
			    if (preg_match($re, $item['uid'])) {
				$item['axis'] = $axis;
				break;
			    }
			}
		    }
		}
	    } else {
		$item["axis"] = $axis_opt;
	    }
	}
    }
 }

 function CheckGroup(LOGGROUP &$grp = NULL) {
    if ($grp) {
	if (($this->group_class)&&(!$grp instanceof $this->group_class))
	    throw Exception(translate("Invalid LOGGROUP supplied"));
    } else {
	$grinfo = $this->req->GetGroupInfo();
	$grp = $this->CreateGroup($grinfo);
    }
    return $grp;
 }

 function CreateGroup(array &$ginfo = NULL, $flags = 0) {
    if (!$ginfo) $ginfo = $this->req->GetGroupInfo();

    if ($this->group_class) return new $this->group_class($ginfo, $this, $flags);
    return new LOGGROUP($ginfo, $this, $flags);
 }

 function CreateJointGroup($flags = 0) {
    throw new ADEIException(translate("Only virtual readers are supporting joint groups"));
 }

 function CreateMask(LOGGROUP $grp = NULL, array &$minfo = NULL, $flags = 0) {
    if (!is_array($minfo)) {
	if ($this->req instanceof GROUPRequest)
	    $minfo = $this->req->GetMaskInfo();
    }
    //$grp = $this->CheckGroup($grp);
    
    return new MASK($minfo);
 }
 
 function CreateInterval(LOGGROUP $grp = NULL, array &$iinfo = NULL, $flags = 0) {
    if (!$iinfo) {
	if ($this->req instanceof DATARequest)
	    $iinfo = $this->req->GetIntervalInfo();
    }
    $grp = $this->CheckGroup($grp);
    
    $ivl = new INTERVAL($iinfo, $this, $grp, $flags);
    $ivl->Limit($this->start_date, $this->end_date);
    
    return $ivl;
 }

 function CreateCache(LOGGROUP $grp = NULL, $flags = 0) {
    return new CACHE($this->req, $this, $flags);
 }

 function CreateCacheSet(LOGGROUP $grp = NULL, MASK $msk = NULL) {
    $cache = $this->CreateCache($grp);
    return $this->req->CreateSimpleCacheSet($msk, $cache);
 }
 
 function CreateRequestSet(LOGGROUP $grp = NULL, MASK $msk = NULL, $type = "GROUPRequest") {
    return $this->req->CreateSimpleRequestSet($msk, $grp, $type);
 }

 function CreateAxes($flags = 0) {
    return new GRAPHAxes($this->req, $flags);
 }

 function GetGroups() {
    $groups = $this->GetGroupList(); 

    $list = array();
    foreach (array_keys($groups) as $group) {
    	$ginfo = array("db_group" => $group);
	array_push($list, $this->CreateGroup($ginfo));
    }

    return $list;
 }

 function GetGroupSize(LOGGROUP $grp = NULL, $flags = 0) {
    $grp = $this->CheckGroup($grp);

    $params = $this->GetGroupParameters($grp);

    if (!$params['width']) {
	$params['width'] = sizeof($this->GetItemList($grp, $mask = new MASK()));
    }

    return $params['width'];
 }
 
 function HaveData(LOGGROUP $grp = NULL, $from = false, $to = false) {
    return true;
 }

 function PushData(LOGGROUP $grp, $time, $data) {
    throw new ADEIException(get_class($this) . ". PushData is not implemented");
 }
 
 function RemoveData(LOGGROUP $grp, $time) {
    throw new ADEIException(get_class($this) . ". RemoveData is not implemented");
 }
    
 /* DS: check for invalid columns number */
 function Clean(&$lg, $from = false, $to = false) {
    $data = $this->GetData($lg, $from, $to);
    foreach ($data as $t => $value) {
	if (sizeof($value)==0) {
	    $this->RemoveData($lg, $t);
	}
	
	$check = true;
	foreach ($value as $v) {
	    if ($v) {
		$check = false;
		break;
	    }
	}
	
	if ($check) {
	    $this->RemoveData($lg, $t);
	}
    }
 }

 function GetOption($prop, $default = NULL) {
    return $this->req->GetOption($prop, $default);
 }
 
 function GetGroupOption(LOGGROUP $grp = NULL, $prop, $default = NULL) {
    return $this->req->GetGroupOption($prop, $grp, $default);
 }

 function GetGroupParameters(LOGGROUP $grp, $prm = false) {
    global $DEFAULT_MISSING_VALUE;
    
    if (!$this->groups[$grp->gid]) $this->groups[$grp->gid] = array();
    $group = &$this->groups[$grp->gid];
/*
    switch ($prm) {
    }
*/    
    return $group;
 }

 function Backup(&$binfo) {
    throw new ADEIException(get_class($this) . ". Backup is not implemented");
 }

 function CreateDataFilter(LOGGROUP $grp = NULL, MASK $mask = NULL, $resample = false, $limit = 0, $additional_filter = NULL) {
    $grp = $this->CheckGroup($grp);

    $filters = $this->GetGroupOption($grp, "data_filters");
    if ((!$filters)&&(isset($this->data_filters))) $filters = $this->data_filters;
    
    if ($resample === false) {
	$resample = $this->GetGroupOption($grp, "resample", 0);
    }  

    if (($filters)||($mask)||($resample)||($limit)) {
	if ($additional_filter) $filter = new READER_SUPERDataFilter($additional_filter, $this, $grp, $filters, $mask, $resample, $limit);
	else $filter = new READER_DATAFilter($this, $grp, $filters, $mask, $resample, $limit);
	
	return $filter;
    }
    
    return NULL;
 }

 function GetData(LOGGROUP $grp = NULL, $from = false, $to = false, MASK $mask = NULL, $resample = false, $limit = 0) {
    $grp = $this->CheckGroup($grp);
    $filter = $this->CreateDataFilter($grp, $mask, $resample, $limit);
    return $this->GetFilteredData($grp, $from, $to, $filter);
 }

 function GetFilteredData(LOGGROUP $grp = NULL, $from = false, $to = false, DATAFilter $filter = NULL, $filter_data = NULL) {
    $grp = $this->CheckGroup($grp);
    if (($filter)||($filter = $this->CreateDataFilter($grp))) {
	return $filter->ProcessReaderData($this, $grp, $from, $to, $filter_data);
    }

    return $this->GetRawData($grp, $from, $to, $filter, $filter_data);
 }

/*
 function GetIntervalData(LOGGROUP $grp = NULL, INTERVAL $ivl, DATAFilter $filter = NULL, $filter_data = NULL) {
    $from = $ivl->GetWindowStart();
    $to = $ivl->GetWindowEnd();
    $this->GetFilteredData($grp, $ivl, $filter, $filter_data);
 }
*/

 function CreateAlarmFilter(LOGGROUP $grp = NULL, MASK $mask = NULL, $mode = 0, $severity = false, $limit = false, $additional_filter = NULL) {
    $grp = $this->CheckGroup($grp);

    $filters = $this->GetGroupOption($grp, "alarm_filters");
    if ((!$filters)&&(isset($this->alarm_filters))) $filters = $this->alarm_filters;

    if ($severity === false) {
	if (isset($this->req->props['severity'])) {
	    $severity = $this->req->props['severity'];
	} else {
	    $severity = $this->req->GetOption("alarm_severity", 0);
	}
    }

    if ($limit === false) {
	if (is_numeric($this->req->props['limit'])) {
	    $limit = $this->req->props['limit'];
	} else {
	    $limit = 0;
	}
    }
    
    if (($mask == NULL)&&($this->req->props['alarm_mask'])) {
	$mask = new MASK($mask_info = array(
	    "db_mask" => $this->req->props['alarm_mask']
	));
    }

    if (($filters)||($mask)||($mode)||($limit)||($severity)) {
	if ($additional_filter) $filter = new READER_SUPERFilter($additional_filter, $this, $grp, $filters, $mask, 0, $limit);
	else $filter = new READER_DATAFilter($this, $grp, $filters, $mask, 0, $limit);
	
	$filter->SetProperty("alarm_list_mode", $mode);
	$filter->SetProperty("alarm_severity", $severity);
	
	return $filter;
    }
 }

 function GetRawAlarms(LOGGROUP $grp = NULL, $from = false, $to = false, DATAFilter $filter = NULL, &$filter_data = NULL) {
    return new NOData(); 
 }

 function GetCurrentAlarms(LOGGROUP $grp = NULL, MASK $mask = NULL, $severity = false) {
    $grp = $this->CheckGroup($grp);
    $filter = $this->CreateAlarmFilter($grp, $mask = NULL, READER::ALARM_MODE_ACTIVE_LIST, $severity);
    return $this->GetRawAlarms($grp, $from, $to, $filter);
 }
 
 function GetAlarmsDetailed(LOGGROUP $grp = NULL, $from = false, $to = false, MASK $mask = NULL, $severity = false, $limit = false) {
    $grp = $this->CheckGroup($grp);
    $filter = $this->CreateAlarmFilter($grp, $mask, READER::ALARM_MODE_FULL_LIST, $severity, $limit);
    return $this->GetRawAlarms($grp, $from, $to, $filter);
 }

 function GetAlarms(LOGGROUP $grp = NULL, $from = false, $to = false, MASK $mask = NULL, $mode = 0, $severity = false, $limit = false) {
    $grp = $this->CheckGroup($grp);
    $filter = $this->CreateAlarmFilter($grp, $mask, $mode, $severity, $limit);
    return $this->GetRawAlarms($grp, $from, $to, $filter);
 }
 
 function GetControls(LOGGROUP $grp = NULL, MASK $mask = NULL) {
    $grp = $this->CheckGroup($grp);
	
	/* If NULL is passed for mask, we need to construct common mask here, 
	otherwise GetItemList may use reader-specific 'all' mask, and GetData 
	will surely return all the data */
	
    if (!$mask) $mask = new MASK($info = array());
    
    $info = $this->GetItemList($grp, $mask, REQUEST::CONTROL|REQUEST::NEED_INFO);
    $data = $this->GetData($grp, false, false, $mask, false, -1);


    $res = array();
    
    $data->rewind();
    if ($data->valid()) {
	$row = $data->current();
	$timestamp = $data->key();
	
	foreach ($row as $key => $value) {
	    $res[$key] = array(
		'value' => $value,
		'timestamp' => $timestamp,
		'verified' => $timestamp,
		'obtained' => gettimeofday(true),
		'db_server' => $this->srvid,
		'db_name' => $this->dbname,
		'db_group' => $grp->gid,
		'channel_id' => $info[$key]['id'],
		'channel_name' => $info[$key]['name']
	    );

	    if ($info[$key]['sampling_rate']) {
		$info[$key]['sampling_rate'] = $info[$key]['sampling_rate'];
	    }
	    if ($info[$key]['uid']) {
		$info[$key]['uid'] = $info[$key]['uid'];
	    }
	}
    }
    unset($data);
    
    return $res;
 }

 protected function ExportData(DATAHandler $h = NULL, LOGGROUP $grp, MASK $mask, INTERVAL $ivl = NULL, $resample = 0, array &$names, $opts = 0) {
    global $DEFAULT_MISSING_VALUE;

    if (!$h) $h = new CSVHandler();

    $filter = $this->CreateDataFilter($grp, $mask, $resample, $ivl->GetItemLimit());
    if (!$h->nullwriter) {
	$filter->AddFilter(new NULLCorrector($this->GetGroupOption($grp, "null_value", $DEFAULT_MISSING_VALUE)));
    }
    
    $data = $this->GetFilteredData($grp, $ivl->GetWindowStart(), $ivl->GetWindowEnd(), $filter);

    $columns = sizeof($names);
    
    $h->Start($columns);
    $h->DataHeaders($names);

    foreach ($data as $time => $row) {
	$h->DataVector($time, $row);
    }
    
    $h->End();
 }
 
 function Export(DATAHandler $h = NULL, LOGGROUP $grp = NULL, MASK $mask = NULL, INTERVAL $ivl = NULL, $resample = 0, $opts = 0) {
    $grp = $this->CheckGroup($grp);
    if (!$mask) $mask = $this->CreateMask($grp, $minfo = array());

    $names = $this->GetItemList($grp, $mask);
    return $this->ExportData($h, $grp, $mask, $ivl, $resample, $names, $opts);

 }

 function ExportCSV(STRINGHandler $h = NULL, LOGGROUP $grp = NULL, MASK $mask = NULL, INTERVAL $ivl = NULL, $resample = 0, $opts = 0) {
    return $this->Export(new CSVHandler($h), $grp, $msk, $ivl, $resample, $opts);
 }
}



class NULLReader implements READERInterface {
    function __construct(&$props) {}

    function GetDatabaseList($flags = 0) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }

    function GetGroupList($flags = 0) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }
    
    function GetGroupInfo(LOGGROUP $grp = NULL, $flags = 0) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }
    
    function GetItemList(LOGGROUP $grp = NULL, MASK $mask = NULL, $flags = 0) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }

    function GetExperimentList($flags = 0) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }
    
    function GetMaskList(LOGGROUP $grp = NULL, $flags = 0) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }
    
    function CreateGroup(array &$ginfo = NULL, $flags = 0) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }
    
    function CreateInterval(LOGGROUP $group = NULL, array &$iinfo = NULL, $flags = 0) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }
    
    function CreateMask(LOGGROUP $grp = NULL, array &$minfo = NULL, $flags = 0) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }
    
    function GetGroups() {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }
    
    function GetGroupSize(LOGGROUP $grp = NULL, $flags = 0) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }
    
    function ImportUnixTime($unix_time) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }
    
    function ExportUnixTime($db_time) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }

    function HaveData(LOGGROUP $grp = NULL, $from = false, $to = false) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }
    
    function GetData(LOGGROUP $grp = NULL, $from = false, $to = false, MASK $mask = NULL, $resample = 0, $limit = 0) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }

    function GetRawData(LOGGROUP $grp = NULL, $from = false, $to = false, DATAFilter $filter = NULL, &$filter_data = NULL) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }

    function GetAlarms(LOGGROUP $grp = NULL, $from = false, $to = false, MASK $mask = NULL, $mode = 0, $severity = false, $limit = false) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }
    
    function GetRawAlarms(LOGGROUP $grp = NULL, $from = false, $to = false, DATAFilter $filter = NULL, &$filter_data = NULL) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }

    function GetFilteredData(LOGGROUP $grp = NULL, $from = false, $to = false, DATAFilter $filter = NULL, $filter_data = NULL) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }
    
    function PushData(LOGGROUP $grp, $time, $data) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }
    
    function RemoveData(LOGGROUP $grp, $time) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }
    
    function Backup(&$binfo) {
	throw new ADEIException(translate("%s is not implemented", get_class($this) . ". " . __METHOD__));
    }
}

class NOData implements Iterator {
 function rewind() {}
 function next() {}
 function valid() {
    return false;
 }
 function key() {
    return false;
 }
 function current() {
    return false;
 }
}

require($ADEI_ROOTDIR . "/classes/readers/dbreader.php");

?>