<?php

require_once($GLOBALS['ADEI_ROOTDIR'] . "/classes/readers/zeus.php");

class ZEUSController extends ZEUS /*implements CONTROLInterface*/ {

 function GetGroupInfo(LOGGROUP $grp = NULL, $flags = 0) {
    if (!($flags&REQUEST::CONTROL)) return parent::GetGroupInfo($grp, $flags);
    
    if ($flags&REQUEST::NEED_INFO)
	$req_cols = "bid, name, length, ";
    else
	$req_cols = "bid, name";


    if ($grp)
        $res = $this->db->Query("SELECT $req_cols FROM blocks WHERE bid=" . $grp->gid);
    else
        $res = $this->db->Query("SELECT $req_cols FROM blocks");


    $groups = array();
    foreach ($res as $row) {
        $gid = $row['bid'];

	$groups[$gid] = array(
		'gid' => $gid,
		'name' => $row['name']
	);
	
	if (($flags&REQUEST::NEED_INFO)&&($flags&REQUEST::NEED_ITEMINFO)) {
	    $ginfo = array("db_group" => $gid);
	    $grzeus = $this->CreateGroup($ginfo);
	    $groups[$gid]['items'] = $this->GetItemList($grzeus, NULL, $flags);
	}
    }

    return $grp?$groups[$grp->gid]:$groups;
 }

 public function GetItemList(LOGGROUP $grp = NULL, MASK $mask = NULL, $flags = 0) {
    if (!($flags&REQUEST::CONTROL)) return parent::GetItemList($grp, $mask, $flags);

    $grp = $this->CheckGroup($grp);
    if (!$mask) $mask = $this->CreateMask($grp);

    $uid = $this->opts->Get('control_uids', false);

    $items = array();
    

    $bid = $grp->gid;
    $resp = $this->db->Query("SELECT length, name, itemnames, oid FROM blocks WHERE bid=$bid");
    if ($resp) $res = $resp->fetch();
    else $res = false;
    unset($resp);
    	
    if (!$res) return $items;
    
    $oid = $res['oid'];
    $allow_read = true;
    $allow_write = false;
    $must_update = 0;
    
    $resp = $this->db->Query("SELECT mode, must_update FROM opc WHERE oid=$oid");
    if ($resp) {
	$opcres = $resp->fetch();
	if ($opcres) {
	    switch ($opcres['mode']) {
		case 1:			// write only
		    $allow_read = false;
		case 2:			// read-write
		    $allow_write = true;
	    }
	    $must_update = $opcres['must_update'];
	}
    }
    
    $names = preg_split("/\r?\n/", $res['itemnames']);

    for ($rpos = 0, $pos = 0; $pos < $res['length']; $pos++) {
	if (!$mask->Check($pos)) continue;
	    
	$items[$rpos] = array(
	    "id" => $pos,
	    "group" => $res['name'],
	    "name" =>  $names[$pos],
	    "read" => $allow_read,
	    "write" => $allow_write,
	    "sampling_rate" => $must_update
	);
	    
	
	if (preg_match("/[\w\d]/", $items[$rpos]["name"])) {
	    if ($uid) {
		if (($uid === true)||(preg_match($uid, $items[$rpos]["name"]))) {
		    $items[$rpos]["uid"] = $items[$rpos]["name"];
		}
	    }
	} else {
	    $items[$rpos]["name"] = "item" . $pos;
	}
	
        $rpos++;
    }

    unset($names);
    
    return $items;
 }


 function GetGroupSize(LOGGROUP $grp = NULL, $flags = 0) {
    if (!($flags&REQUEST::CONTROL)) return parent::GetGroupSize($grp, $flags);

    $grp = $this->CheckGroup($grp);

    $params = $this->GetGroupParameters($grp);
    if ($params['width']) return $params['width'];

    $size = 0;
    try {
	$resp = $this->db->Query("SELECT length FROM blocks WHERE bid=" . $grp->gid);
	if ($resp) $res = $resp->fetch();
	else $res = false;
	
	if ($res) $size = $res['length'];
    } catch (PDOException $e) {
	throw new Exception($e->getMessage());
    }
    
    $params['width'] = $size;
    
    return $size;
 }

/*
 function GetControls(LOGGROUP $grp = NULL) {
 }
 
 function SetControls(LOGGROUP $grp = NULL) {
 } 
*/

/*
 public function GetControlGroupList($flags = 0) {
 }
 
 
 public function GetControlValues(CGROUP $grp = NULL, MASK $mask = NULL) {
 }
 
 public function SetControlValues(CGROUP $grp = NULL, MASK $mask = NULL, &$values) {
 }
*/
}



?>